@@ -90,7 +90,6 @@ import org.apache.cxf.jaxrs.model.URITemplate;
 import org.apache.cxf.jaxrs.provider.AbstractConfigurableProvider;
 import org.apache.cxf.jaxrs.provider.ProviderFactory;
 import org.apache.cxf.message.Message;
-import org.apache.cxf.message.MessageImpl;
 import org.apache.cxf.transport.http.AbstractHTTPDestination;
 
 public final class JAXRSUtils {
@@ -135,7 +134,6 @@ public final class JAXRSUtils {
         return values;
     }
     
-    @SuppressWarnings("unchecked")
     public static List<MediaType> getProviderConsumeTypes(MessageBodyReader provider) {
         String[] values = getUserMediaTypes(provider, "getConsumeMediaTypes");
         
@@ -164,17 +162,6 @@ public final class JAXRSUtils {
         return supportedMimeTypes;
     }
     
-    public static ClassResourceInfo findSubResourceClass(ClassResourceInfo resource,
-                                                         Class subResourceClassType) {
-        for (ClassResourceInfo subCri : resource.getSubClassResourceInfo()) {
-            if (subCri.getResourceClass().isAssignableFrom(subResourceClassType)) {
-                return subCri;
-            }
-        }
-        return null;
-    }
-
-    
     @SuppressWarnings("unchecked")
     public static void handleSetters(OperationResourceInfo ori,
                                      Object requestObject,
@@ -237,7 +224,6 @@ public final class JAXRSUtils {
             return firstEntry.getKey();
         }
         
-        
         return null;
     }
 
@@ -252,20 +238,34 @@ public final class JAXRSUtils {
                 new OperationResourceInfoComparator());
         MediaType requestType = requestContentType == null 
                                 ? ALL_TYPES : MediaType.valueOf(requestContentType);
+        
+        int pathMatched = 0;
+        int methodMatched = 0;
+        int consumeMatched = 0;
+        int produceMatched = 0;
+        
         for (MediaType acceptType : acceptContentTypes) {
             for (OperationResourceInfo ori : resource.getMethodDispatcher().getOperationResourceInfos()) {
-                
                 URITemplate uriTemplate = ori.getURITemplate();
                 MultivaluedMap<String, String> map = cloneMap(values);
                 if (uriTemplate != null && uriTemplate.match(path, map)) {
                     if (ori.isSubResourceLocator()) {
                         candidateList.put(ori, map);
-                    } else if (ori.getHttpMethod().equalsIgnoreCase(httpMethod)
-                               && matchMimeTypes(requestType, acceptType, ori)) {
+                    } else {
                         String finalGroup = map.getFirst(URITemplate.FINAL_MATCH_GROUP);
                         if (finalGroup == null || StringUtils.isEmpty(finalGroup)
                             || finalGroup.equals("/")) {
-                            candidateList.put(ori, map);    
+                            pathMatched++;
+                            boolean mMatched = ori.getHttpMethod().equalsIgnoreCase(httpMethod);
+                            boolean cMatched = matchConsumeTypes(requestType, ori);
+                            boolean pMatched = matchProduceTypes(acceptType, ori);
+                            if (mMatched && cMatched && pMatched) {
+                                candidateList.put(ori, map);    
+                            } else {
+                                methodMatched = mMatched ? methodMatched + 1 : methodMatched;
+                                produceMatched = pMatched ? produceMatched + 1 : produceMatched;
+                                consumeMatched = cMatched ? consumeMatched + 1 : consumeMatched;
+                            }
                         }
                     }
                 }
@@ -279,10 +279,28 @@ public final class JAXRSUtils {
             }
         }
 
-        return null;
+        int status = pathMatched == 0 ? 404 : methodMatched == 0 ? 405 
+                     : consumeMatched == 0 ? 415 : produceMatched == 0 ? 406 : 404;
+        String name = resource.isRoot() ? "NO_OP_EXC" : "NO_SUBRESOURCE_METHOD_FOUND";
+        org.apache.cxf.common.i18n.Message errorMsg = 
+            new org.apache.cxf.common.i18n.Message(name, 
+                                                   BUNDLE, 
+                                                   path,
+                                                   requestType.toString(),
+                                                   convertTypesToString(acceptContentTypes));
+        LOG.severe(errorMsg.toString());
+        
+        throw new WebApplicationException(status);
+        
     }    
 
-    
+    private static String convertTypesToString(List<MediaType> types) {
+        StringBuilder sb = new StringBuilder();
+        for (MediaType type : types) {
+            sb.append(type.toString()).append(',');
+        }
+        return sb.toString();
+    }
     
     public static List<MediaType> getConsumeTypes(ConsumeMime cm) {
         return cm == null ? Collections.singletonList(ALL_TYPES)
@@ -449,12 +467,14 @@ public final class JAXRSUtils {
         
         CookieParam cookie = AnnotationUtils.getAnnotation(anns, CookieParam.class);
         if (cookie != null) {
-            result = processCookieParam(message, cookie.value(), parameterClass, genericParam, defaultValue);
+            result = processCookieParam(message, cookie.value(), parameterClass, genericParam,
+                                        defaultValue);
         } 
         
         HeaderParam hp = AnnotationUtils.getAnnotation(anns, HeaderParam.class);
         if (hp != null) {
-            result = processHeaderParam(message, hp.value(), parameterClass, genericParam, defaultValue);
+            result = processHeaderParam(message, hp.value(), parameterClass, genericParam, 
+                                        defaultValue);
         } 
 
         return result;
@@ -477,8 +497,13 @@ public final class JAXRSUtils {
                 }
             }
             
+            String basePath = HttpUtils.getOriginalAddress(m);
+            
             if ("".equals(key)) {
-                return InjectionUtils.handleBean(pClass, params, false);
+                return InjectionUtils.handleBean(pClass, 
+                                                 params, 
+                                                 ParameterType.MATRIX,
+                                                 basePath);
             } else {
                 List<String> values = params.get(key);
                 return InjectionUtils.createParameterObject(values, 
@@ -486,23 +511,20 @@ public final class JAXRSUtils {
                                                             genericType,
                                                             defaultValue,
                                                             false,
-                                                            false,
-                                                            true);
+                                                            ParameterType.MATRIX,
+                                                            basePath);
             }
         }
         
         return null;
     }
     
-    
-    
     public static MultivaluedMap<String, String> getMatrixParams(String path, boolean decode) {
         int index = path.indexOf(';');
         return index == -1 ? new MetadataMap<String, String>()
                            : JAXRSUtils.getStructuredParams(path.substring(index + 1), ";", decode);
     }
     
-    @SuppressWarnings("unchecked")
     private static Object processHeaderParam(Message m, 
                                              String header, 
                                              Class<?> pClass, 
@@ -510,14 +532,14 @@ public final class JAXRSUtils {
                                              String defaultValue) {
         
         List<String> values = new HttpHeadersImpl(m).getRequestHeader(header);
-        
+        String basePath = HttpUtils.getOriginalAddress(m);
         return InjectionUtils.createParameterObject(values, 
                                                     pClass, 
                                                     genericType,
                                                     defaultValue,
                                                     false,
-                                                    false,
-                                                    false);
+                                                    ParameterType.HEADER,
+                                                    basePath);
              
         
     }
@@ -536,7 +558,12 @@ public final class JAXRSUtils {
         if (pClass.isAssignableFrom(Cookie.class)) {
             return Cookie.valueOf(value.length() == 0 ? defaultValue : value);
         }
-        return value.length() > 0 ? InjectionUtils.handleParameter(value, pClass, false) 
+        
+        String basePath = HttpUtils.getOriginalAddress(m);
+        return value.length() > 0 ? InjectionUtils.handleParameter(value, 
+                                                                   pClass, 
+                                                                   ParameterType.COOKIE,
+                                                                   basePath) 
                                   : defaultValue;
     }
     
@@ -611,18 +638,19 @@ public final class JAXRSUtils {
                                            String defaultValue,
                                            boolean  decoded) {
         
+        String basePath = HttpUtils.getOriginalAddress(m);
         String parameterName = uriParamAnnotation.value();
         if ("".equals(parameterName)) {
-            return InjectionUtils.handleBean(paramType, values, true);
+            return InjectionUtils.handleBean(paramType, values, ParameterType.PATH, basePath);
         } else {
             List<String> results = values.get(parameterName);
             return InjectionUtils.createParameterObject(results, 
                                                     paramType, 
                                                     genericType,
                                                     defaultValue,
-                                                    true,
                                                     decoded,
-                                                    true);
+                                                    ParameterType.PATH,
+                                                    basePath);
         }
     }
     
@@ -637,10 +665,11 @@ public final class JAXRSUtils {
                                           boolean decode) {
         String queryName = queryParam.value();
 
+        String basePath = HttpUtils.getOriginalAddress(m);
         if ("".equals(queryName)) {
             return InjectionUtils.handleBean(paramType, 
                                              new UriInfoImpl(m, null).getQueryParameters(),
-                                             false);
+                                             ParameterType.QUERY, basePath);
         } else {
             List<String> results = getStructuredParams((String)m.get(Message.QUERY_STRING),
                                        "&",
@@ -651,8 +680,7 @@ public final class JAXRSUtils {
                                                         genericType,
                                                         defaultValue,
                                                         false,
-                                                        false,
-                                                        false);
+                                                        ParameterType.QUERY, basePath);
              
         }
     }
@@ -709,7 +737,6 @@ public final class JAXRSUtils {
                                          parameterAnnotations,
                                          type,
                                          m);
-            // TODO : make the exceptions
             if (provider != null) {
                 try {
                     HttpHeaders headers = new HttpHeadersImpl(m);
@@ -717,15 +744,19 @@ public final class JAXRSUtils {
                               targetTypeClass, parameterType, parameterAnnotations, contentType,
                               headers.getRequestHeaders(), is);
                 } catch (IOException e) {
-                    e.printStackTrace();
-                    throw new RuntimeException("Error deserializing input stream into target class "
-                                               + targetTypeClass.getSimpleName() 
-                                               + ", content type : " + contentType);
+                    String errorMessage = "Error deserializing input stream into target class "
+                                          + targetTypeClass.getSimpleName() 
+                                           + ", content type : " + contentType;
+                    LOG.severe(errorMessage);
+                    throw new WebApplicationException(Response.Status.INTERNAL_SERVER_ERROR);
                 }    
             } else {
-                throw new RuntimeException("No message body reader found for target class "
-                                           + targetTypeClass.getSimpleName() 
-                                           + ", content type : " + contentType);
+                String errorMessage = new org.apache.cxf.common.i18n.Message("NO_MSG_READER",
+                                                       BUNDLE,
+                                                       targetTypeClass.getSimpleName(),
+                                                       contentType).toString();
+                LOG.severe(errorMessage);
+                throw new WebApplicationException(Response.Status.UNSUPPORTED_MEDIA_TYPE);
             }
         }
 
@@ -734,6 +765,18 @@ public final class JAXRSUtils {
 
     
 
+    public static boolean matchConsumeTypes(MediaType requestContentType, 
+                                            OperationResourceInfo ori) {
+        
+        return intersectMimeTypes(ori.getConsumeTypes(), requestContentType).size() != 0;
+    }
+    
+    public static boolean matchProduceTypes(MediaType acceptContentType, 
+                                            OperationResourceInfo ori) {
+        
+        return intersectMimeTypes(ori.getProduceTypes(), acceptContentType).size() != 0;
+    }
+    
     public static boolean matchMimeTypes(MediaType requestContentType, 
                                          MediaType acceptContentType, 
                                          OperationResourceInfo ori) {
@@ -782,12 +825,17 @@ public final class JAXRSUtils {
             for (MediaType userType : userMediaTypes) {
                 if (requiredType.isCompatible(userType) || userType.isCompatible(requiredType)) {
                     
+                    boolean parametersMatched = true;
                     for (Map.Entry<String, String> entry : userType.getParameters().entrySet()) {
                         String value = requiredType.getParameters().get(entry.getKey());
                         if (value != null && !value.equals(entry.getValue())) {
-                            continue;
+                            parametersMatched = false;
+                            break;
                         }
                     }
+                    if (!parametersMatched) {
+                        continue;
+                    }
                     
                     String type = requiredType.getType().equals(MediaType.MEDIA_TYPE_WILDCARD) 
                                       ? userType.getType() : requiredType.getType();
@@ -843,11 +891,10 @@ public final class JAXRSUtils {
     }
     
     @SuppressWarnings("unchecked")
-    public static Response convertFaultToResponse(Throwable ex, String baseAddress) {
+    public static Response convertFaultToResponse(Throwable ex, String baseAddress, Message inMessage) {
         
         ExceptionMapper mapper = 
-            ProviderFactory.getInstance(baseAddress).createExceptionMapper(ex.getClass(),
-                                                                new MessageImpl());
+            ProviderFactory.getInstance(baseAddress).createExceptionMapper(ex.getClass(), inMessage);
         if (mapper != null) {
             Response excResponse = mapper.toResponse(ex);
             if (excResponse != null) {
