@@ -35,21 +35,15 @@ import org.apache.cxf.testutil.common.AbstractBusClientServerTestBase;
 import org.apache.hello_world_rpclit.GreeterRPCLit;
 import org.apache.hello_world_rpclit.SOAPServiceRPCLit;
 import org.junit.BeforeClass;
-import org.junit.Ignore;
 import org.junit.Test;
 
 public class ProviderRPCClientServerTest extends AbstractBusClientServerTestBase {
 
     @BeforeClass
     public static void startServers() throws Exception {
-        assertTrue("server did not launch correctly", launchServer(Server.class, true));
+        assertTrue("server did not launch correctly", launchServer(Server.class));
     }
     
-    //
-    // Skipping this test as of 20081003 - update of released sun saaj in
-    // suns maven repo seems to have broken this test.
-    //
-    @Ignore
     @Test
     public void testSWA() throws Exception {
         SOAPFactory soapFac = SOAPFactory.newInstance();
@@ -65,6 +59,7 @@ public class ProviderRPCClientServerTest extends AbstractBusClientServerTestBase
         AttachmentPart ap2 = msg.createAttachmentPart();
         ap2.setContent("Attachment content - Part 2", "text/plain");
         msg.addAttachmentPart(ap2);
+        msg.saveChanges();
         
         SOAPConnection con = conFac.createConnection();
         URL endpoint = new URL("http://localhost:9008/SOAPServiceProviderRPCLit/SoapPortProviderRPCLit1");
